/* Copyright (c) 2016, Jan Wiśniewski <vuko@hackerspace.pl>
 * 
 * This software is provided 'as-is', without any express or implied
 * warranty. In no event will the authors be held liable for any damages
 * arising from the use of this software.
 * 
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 * 
 * 1. The origin of this software must not be misrepresented; you must not
 *    claim that you wrote the original software. If you use this software
 *    in a product, an acknowledgment in the product documentation would be
 *    appreciated but is not required.
 * 2. Altered source versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 */
#include"fifo.h"

void fifoInit(fifo *f, uint8_t *addr, uint16_t size){
    f->first = 0;
    f->last = 0;
    f->size = size;
    f->data = addr;
};

inline uint8_t fifoEmpty(fifo *f){
    return f->first == f->last;
}

inline uint8_t fifoFull(fifo *f){
    return f->first == (f->last+1)%f->size;
}

void fifoPush(fifo *f, uint8_t d){
    if(!fifoFull(f)){
        f->data[f->last] = d;
        f->last = (f->last + 1)%f->size;
    }
} 

uint8_t fifoPop(fifo *f){
    uint16_t first = f->first;
    if(!fifoEmpty(f)){
        f->first = (f->first + 1)%f->size;
        return f->data[first];
    } else {
        return 0x00;
    }
}
